# include	<math.h>
# include	"Utility.h"
# include	"CIE.h"

namespace	ColorimetrySpace
{

XYZ::XYZ(float thisX, float thisY, float thisZ) noexcept:
	X(thisX), Y(thisY), Z(thisZ)
	{
	}

float	XYZ::GetX(void) const noexcept
	{
	return X;
	}

float	XYZ::GetY(void) const noexcept
	{
	return Y;
	}

float	XYZ::GetZ(void) const noexcept
	{
	return Z;
	}

float	XYZ::GetSum(void) const noexcept
	{
	return X + Y + Z;
	}

void	XYZ::SetX(float Value) noexcept
	{
	X = Value;
	}

void	XYZ::SetY(float Value) noexcept
	{
	Y = Value;
	}

void	XYZ::SetZ(float Value) noexcept
	{
	Z = Value;
	}

xyzcc::xyzcc(const XYZ& Argument) noexcept: XYZ(Argument)
	{
	float	Divisor;

	Divisor = GetSum();
	SetX(GetX() / Divisor);
	SetY(GetY() / Divisor);
	SetZ(GetZ() / Divisor);
	}

xyY::xyY(float thisx, float thisy, float thisYBig) noexcept: x(thisx), y(thisy),
	YBig(thisYBig)
	{
	}

xyY::xyY(const XYZ& Argument) noexcept
	{
	float	Divisor;

	Divisor = Argument.GetSum();
	x = Argument.GetX() / Divisor;
	y = Argument.GetY() / Divisor;
	YBig = Argument.GetY();
	}

float	xyY::Getx(void) const noexcept
	{
	return x;
	}

float	xyY::Gety(void) const noexcept
	{
	return y;
	}

float	xyY::GetYBig(void) const noexcept
	{
	return YBig;
	}

Labstar::Labstar(float thisLstar, float thisastar, float thisbstar) noexcept:
	Lstar(thisLstar), astar(thisastar), bstar(thisbstar)
	{
	}

Labstar::Labstar(const XYZ& Argument, const XYZ& Argument_n) noexcept
	{
	float	y_n;
	XYZ	XYZ_n3(Compute_n3(Argument.GetX() / Argument_n.GetX()),
			Compute_n3(y_n = Argument.GetY() / Argument_n.GetY()),
			Compute_n3(Argument.GetZ() / Argument_n.GetZ()));

	Lstar = y_n > 0.008856? 116 * XYZ_n3.GetY() - 16: 903.3 * y_n;
	astar = 500 * (XYZ_n3.GetX() - XYZ_n3.GetY());
	bstar = 200 * (XYZ_n3.GetY() - XYZ_n3.GetZ());
	}

float	Labstar::GetLstar(void) const noexcept
	{
	return Lstar;
	}

float	Labstar::Getastar(void) const noexcept
	{
	return astar;
	}

float	Labstar::Getbstar(void) const noexcept
	{
	return bstar;
	}

LHabCabstar::LHabCabstar(float thisLstar, float thisHab, float thisCabstar)
	noexcept: Lstar(thisLstar), Hab(thisHab), Cabstar(thisCabstar)
	{
	}

LHabCabstar::LHabCabstar(const XYZ& Argument, const XYZ& Argument_n) noexcept
	{
	*this = LHabCabstar(Labstar(Argument, Argument_n));
	}

LHabCabstar::LHabCabstar(const Labstar& Argument) noexcept:
	Lstar(Argument.GetLstar()),
	Hab(Arctan(Argument.Getbstar(), Argument.Getastar())),
	Cabstar(sqrt(Square(Argument.Getastar()) + Square(Argument.Getbstar())))
	{
	}

float	LHabCabstar::GetLstar(void) const noexcept
	{
	return Lstar;
	}

float	LHabCabstar::GetHab(void) const noexcept
	{
	return Hab;
	}

float	LHabCabstar::GetCabstar(void) const noexcept
	{
	return Cabstar;
	}

uvapostr::uvapostr(float thisuapostr, float thisvapostr) noexcept:
	uapostr(thisuapostr), vapostr(thisvapostr)
	{
	}

uvapostr::uvapostr(const XYZ& Argument) noexcept
	{
	float	Denominator;

	Denominator = Argument.GetX() + 15 * Argument.GetY() +
		3 * Argument.GetZ();
	uapostr = 4 * Argument.GetX() / Denominator;
	vapostr = 9 * Argument.GetY() / Denominator;
	}

float	uvapostr::Getuapostr(void) const noexcept
	{
	return uapostr;
	}

float	uvapostr::Getvapostr(void) const noexcept
	{
	return vapostr;
	}

Luvstar::Luvstar(float thisLstar, float thisustar, float thisvstar) noexcept:
	Lstar(thisLstar), ustar(thisustar), vstar(thisvstar)
	{
	}

Luvstar::Luvstar(const XYZ& Argument, const XYZ& Argument_n) noexcept
	{
	float	ucc;
	float	vcc;
	float	xcc;
	float	ycc;
	float	ucn;
	float	vcn;
	float	xcn;
	float	ycn;
	float	Yn;
	float	Yn3;
	float	Denominator;
	xyzcc	Localxyzcc(Argument);

	xcc = Localxyzcc.GetX();
	ycc = Localxyzcc.GetY();
	Denominator = -2 * xcc + 12 * ycc + 3;
	ucc = 4 * xcc / Denominator;
	vcc = 9 * ycc / Denominator;
	Denominator = Argument_n.GetSum();
	xcn = Argument_n.GetX() / Denominator;
	ycn = Argument_n.GetY() / Denominator;
	Denominator = -2 * xcn + 12 * ycn + 3;
	ucn = 4 * xcn / Denominator;
	vcn = 9 * ycn / Denominator;
	Yn = Argument.GetY() / Argument_n.GetY();
	Yn3 = Compute_n3(Yn);
	Lstar = Yn > 0.008856? 116 * Yn3 - 16: 903.3 * Yn;
	ustar = 13 * Lstar * (ucc - ucn);
	vstar = 13 * Lstar * (vcc - vcn);
	}

float	Luvstar::GetLstar(void) const noexcept
	{
	return Lstar;
	}

float	Luvstar::Getustar(void) const noexcept
	{
	return ustar;
	}

float	Luvstar::Getvstar(void) const noexcept
	{
	return vstar;
	}

LHuvCuvstar::LHuvCuvstar(float thisLstar, float thisHuv, float thisCuvstar)
	noexcept: Lstar(thisLstar), Huv(thisHuv), Cuvstar(thisCuvstar)
	{
	}

LHuvCuvstar::LHuvCuvstar(const XYZ& Argument, const XYZ& Argument_n) noexcept
	{
        *this = LHuvCuvstar(Luvstar(Argument, Argument_n));
	}

LHuvCuvstar::LHuvCuvstar(const Luvstar& Argument) noexcept:
	Lstar(Argument.GetLstar()),
	Huv(Arctan(Argument.Getvstar(), Argument.Getustar())),
	Cuvstar(sqrt(Square(Argument.Getustar()) + Square(Argument.Getvstar())))
	{
	}

float	LHuvCuvstar::GetLstar(void) const noexcept
	{
	return Lstar;
	}

float	LHuvCuvstar::GetHuv(void) const noexcept
	{
	return Huv;
	}

float	LHuvCuvstar::GetCuvstar(void) const noexcept
	{
	return Cuvstar;
	}

float	Compute_n3(float Value) noexcept
	{
	if (Value < 0.008856)
		return 7.787 * Value + 16.0 / 116.0;
	return pow(Value, 1.0F / 3.0F);
	}

CIEColorSystems::CIEColorSystems(const XYZ& Argument) noexcept:
	XYZValue(Argument), xyzccValue(Argument), xyYValue(Argument)
	{
	}

XYZ	CIEColorSystems::GetXYZ(void) const noexcept
	{
	return XYZValue;
	}

xyzcc	CIEColorSystems::Getxyzcc(void) const noexcept
	{
	return xyzccValue;
	}

xyY	CIEColorSystems::GetxyY(void) const noexcept
	{
	return xyYValue;
	}

CIE76ColorSystems::CIE76ColorSystems(const XYZ& Argument, const XYZ& Argument_n)
	noexcept: LabstarValue(Argument, Argument_n),
	LHabCabstarValue(Argument, Argument_n),
	LuvstarValue(Argument, Argument_n),
	LHuvCuvstarValue(Argument, Argument_n), uvapostrValue(Argument)
	{
	}

Labstar		CIE76ColorSystems::GetLabstar(void) const noexcept
	{
	return LabstarValue;
	}

LHabCabstar	CIE76ColorSystems::GetLHabCabstar(void) const noexcept
	{
	return LHabCabstarValue;
	}

Luvstar		CIE76ColorSystems::GetLuvstar(void) const noexcept
	{
	return LuvstarValue;
	}

LHuvCuvstar	CIE76ColorSystems::GetLHuvCuvstar(void) const noexcept
	{
	return LHuvCuvstarValue;
	}

uvapostr	CIE76ColorSystems::Getuvapostr(void) const noexcept
	{
	return uvapostrValue;
	}

}
