# include	"LMS.h"

namespace	ColorimetrySpace
{

LMS::LMS(float thisL, float thisM, float thisS) noexcept: L(thisL), M(thisM),
	S(thisS)
	{
	}

LMS::LMS(const XYZ& Argument, ColorRepresentationType Representation) noexcept
	{
	switch (Representation)
		{
		case ColorRepresentationType::VOS:
			L = 0.15516 * Argument.GetX() +
				0.54308 * Argument.GetY() -
				0.03287 * Argument.GetZ();
			M = -0.15516 * Argument.GetX() +
				0.45692 * Argument.GetY() +
				0.03287 * Argument.GetZ();
			S = 0.01608 * Argument.GetZ();
			return;
		case ColorRepresentationType::SS2:
			L = 0.68990272 * (0.21057582 * Argument.GetX() +
				0.85509764 * Argument.GetY() -
				0.03969827 * Argument.GetZ());
			M = 0.34832189 * (-0.41707637 * Argument.GetX() +
				1.1772611 * Argument.GetY() +
				0.07862825 * Argument.GetZ());
			S = 0.03710971 * 0.51683501 * Argument.GetZ();
			return;
		case ColorRepresentationType::SS10:
			L = 0.69283932 * (0.21701045 * Argument.GetX() +
				0.83573367 * Argument.GetY() -
				0.0435106 * Argument.GetZ());
			M = 0.34967567 * (-0.42997951 * Argument.GetX() +
				1.20388946 * Argument.GetY() +
				0.0862109 * Argument.GetZ());
			S = 0.0554786 * 0.46579234 * Argument.GetZ();
			return;
		}
	}

float	LMS::GetL(void) const noexcept
	{
	return L;
	}

float	LMS::GetM(void) const noexcept
	{
	return M;
	}

float	LMS::GetS(void) const noexcept
	{
	return S;
	}

lmscc::lmscc(float thisL, float thisM, float thisS) noexcept:
	LMS(thisL, thisM, thisS)
	{
	}

lmscc::lmscc(const LMS& Argument) noexcept: LMS(Argument)
	{
	float	Denominator;

	Denominator = L + M;
	L /= Denominator;
	M /= Denominator;
	S /= Denominator;
	}

lmscc::lmscc(const XYZ& Argument, ColorRepresentationType Representation)
	noexcept: LMS(Argument, Representation)
	{
	lmscc(static_cast<const LMS&>(*this));
	}

lsY::lsY(float thisl, float thiss, float thisY) noexcept: l(thisl), s(thiss),
	Y(thisY)
	{
	}

lsY::lsY(const LMS& Argument) noexcept
	{
	Y = Argument.GetL() + Argument.GetM();
	l = Argument.GetL() / Y;
	s = Argument.GetS() / Y;
	}

lsY::lsY(const XYZ& Argument, ColorRepresentationType Representation) noexcept
	{
	lsY(LMS(Argument, Representation));
	}

float	lsY::Getl(void) const noexcept
	{
	return l;
	}

float	lsY::Gets(void) const noexcept
	{
	return s;
	}

float	lsY::GetY(void) const noexcept
	{
	return Y;
	}

FundamentalColorSystem::FundamentalColorSystem(const LMS& thisLMS) noexcept:
	LMSValue(thisLMS), lmsccValue(LMSValue), lsYValue(LMSValue)
	{
	}

FundamentalColorSystem::FundamentalColorSystem(const XYZ& Argument,
	ColorRepresentationType Representation) noexcept:
	LMSValue(Argument, Representation), lmsccValue(LMSValue),
	lsYValue(LMSValue)
	{
	}

LMS	FundamentalColorSystem::GetLMS(void) const noexcept
	{
	return LMSValue;
	}

lmscc	FundamentalColorSystem::Getlmscc(void) const noexcept
	{
	return lmsccValue;
	}

lsY	FundamentalColorSystem::GetlsY(void) const noexcept
	{
	return lsYValue;
	}

}
